/*
CC BY-NC-SA
(c) 2012 Sascha Kloß

http://www.mikrocontroller.net/topic/218643
*/
#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include "bsb.h"


BSB::BSB(uint8_t rx, uint8_t tx) {
  serial = new SoftwareSerial(rx, tx);

  serial->begin(4800);
  serial->listen();
}

bool BSB::waitForMessage(BSB_MESSAGE* msg) {
  // Read serial data until SOF detected (0xDC)
  while (serial->available() > 0) {
    if ((serial->read() ^ 0xFF) == 0xDC) {
      this->receive(msg);
      serial->flush();
      return true;
    }
  }

  return false;
}

bool BSB::receive(BSB_MESSAGE* msg) {
  // Read rest of the message
  // Wait a bit
  delay(1);
  byte len=0;
  byte buffer[32];

  while (serial->available() > 0) {
    byte read = serial->read() ^ 0xFF;
    
    buffer[len] = read;

    Serial.print(read, HEX);
    Serial.print(" ");
    Serial.println(buffer[len], HEX);

    len++;
    delayMicroseconds(220);
  }

  // Copy buffer to structure for better handling
  return memcpy (msg, &buffer, len);
}

bool BSB::parse(BSB_MESSAGE* msg) {
  // This is where the magic will happen

  // TYP1=7, A2=B0 -> State of heat-pump
  if (msg->TYP1==0x07 && msg->A2==0xB0) {
    switch(msg->DATA[3]) {
      // Störung / Jammed
    case 0x02:
      this->state = jammed;
      break;

      // Aus / Off
    case 0x58:
      this->state = off;
      break;

      // Antifreeze / Frostschutz
    case 0x79:
      this->state = antifreeze;
      break;

      // Begrenzungszeit aktiv / wait time
    case 0x87:
      this->state = waittime;
      break;

      // Abtauen / Unfreezing
    case 0x7A:
      this->state = unfreezing;
      break;

      // Heizbetrieb / Heating
    case 0xE1:
      this->state = heating;
      break;

      // EVU-Sperre / Locked
    case 0x0A:
      this->state = locked;
      break;
    }

    return true;
  }

  // A2=19 --> System state / outside temperature
  if (msg->A2==0x19) {
    T_Outside = GetTempFromData(msg, 0);
    return true;
  }

  // TYP1=02, A2=15  --> Room temperature
  if (msg->TYP1==0x02 && msg->A2==0x15) {
    T_Room = GetTempFromData(msg, 0);
    return true;
  }

  // A2=29 --> Information from expansion modules
  if (msg->A2==0x29) {

    if (msg->SRC==0x84) {
      // Vorlauf / Flow
      T_Flow = GetTempFromData(msg, 0);

      return true;
    }
    else if (msg->SRC==0x83) {
      // TWW / Water
      T_Water = GetTempFromData(msg, 0);

      // Puffer
      T_Puffer = GetTempFromData(msg, 2);

      return true;
    }
  }

  return false;
}

double BSB::GetTempFromData(BSB_MESSAGE* msg, byte DataIndex) {
  // Temperatures are SIGNED INT 16-Bit in width
  int converted = int(msg->DATA[DataIndex] << 8) + int(msg->DATA[DataIndex+1]);

  // Restored integer, now convert to temperature by dividing by 64
  return (float)(converted / 64);
}

